import { Client, Message, TextChannel } from 'discord.js';
import { ImplementableApi } from './implementableApi';

import { once as eventsOnce } from 'events';

namespace DiscordParser {
    export type Config = ImplementableApi.Config & {
        token: string;
        channelsId: {
            idChannelYtb: string;
            idChannelPeerTube: string;
        };
        keyWord: string;
    };

    export type ConstructorPattern = ImplementableApi.Config & {
        botPrefix: string;
        channels: {
            ChannelYtb: TextChannel;
            ChannelPeerTube: TextChannel;
        };
        client: Client;
    };
}

type ChannelsType = {
    ChannelYtb: TextChannel;
    ChannelPeerTube: TextChannel;
};

type TBaseDiscordMessageType = ('addListener' | 'newEntriesNotify') &
    ImplementableApi.TBaseMessageType;

class DiscordParser<
    T extends TBaseDiscordMessageType
> extends ImplementableApi<T> {
    readonly botPrefix: string;
    readonly channels: {
        [key: string]: TextChannel;
        ChannelYtb: TextChannel;
        ChannelPeerTube: TextChannel;
    };
    readonly client: Client;

    constructor(readonly config: DiscordParser.ConstructorPattern) {
        super(config);
        this.botPrefix = config.botPrefix;
        this.channels = config.channels;
        this.client = config.client;
        this.settingEvents();
    }

    static async instanciate<T extends TBaseDiscordMessageType>(
        config: DiscordParser.Config
    ): Promise<DiscordParser<T>> {
        const client = new Client();
        client.login(config.token);
        await eventsOnce(client, 'ready');
        const channels: ChannelsType = {
            ChannelPeerTube: this.getTextChannel(
                client,
                config.channelsId.idChannelPeerTube
            ),
            ChannelYtb: this.getTextChannel(
                client,
                config.channelsId.idChannelYtb
            ),
        };

        return new DiscordParser<T>({
            name: config.name,
            channels: channels,
            client: client,
            botPrefix: config.keyWord,
        });
    }

    private static getTextChannel(client: Client, id: string): TextChannel {
        const channel = client.channels.resolve(id);
        if (!channel || !(channel instanceof TextChannel))
            throw 'Bad token or bad channel id. Be careful, the channel must be a TextChannel';
        return channel;
    }

    public receivedMessage(message: ImplementableApi.Message<T>) {
        switch (message.type) {
            case 'newEntriesNotify':
                this.sendMsgYtb(
                    `New YouTubes entries received :\n${message.rawContent.items.map(
                        (item: any) =>
                            `Author : ${item.author}\nTitle: ${item.title}\nlink: ${item.link}`
                    )}`
                );
            default:
                break;
        }
    }

    private async sendMsgYtb(message: string) {
        const resolvedChannel = await this.channels;
        resolvedChannel.ChannelYtb.send(message);
    }
    private async sendMsgPeerTube(message: string) {
        const resolvedChannel = await this.channels;
        resolvedChannel.ChannelPeerTube.send(message);
    }

    private settingEvents(): void {
        this.client.on('message', (message: Message) => {
            const resolvedChannel = this.channels;
            let id_arr: string[] = [];
            for (const key in this.channels)
                id_arr.push(resolvedChannel[key].id);

            if (this.channels)
                if (id_arr.includes(message.channel.id)) {
                    const msg_splitted = message.content.split(' ');
                    if (msg_splitted[0] === this.botPrefix) {
                        switch (msg_splitted[1]) {
                            case 'add':
                                const send_message: ImplementableApi.Message<TBaseDiscordMessageType> =
                                    {
                                        rawContent: {
                                            address: msg_splitted[2],
                                            user: message.author.toString(),
                                            date: message.createdAt.toUTCString(),
                                        },
                                        type: 'addListener',
                                    };
                                message.channel.send('Ceci est un feedback');
                                this.emit('addListener', send_message);
                        }
                    }
                }
        });
    }
}

export { DiscordParser };
